library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

entity RecordControl is
    Port ( 
	     clk100			: in STD_LOGIC;
        laser_data : in  STD_LOGIC_VECTOR (24 downto 0); -- laser data
        laser_data_ready : in STD_LOGIC; -- start record
        max_count : in STD_LOGIC_VECTOR (31 downto 0); -- number of clock cycles after which recording starts
        is_ringup : in  STD_LOGIC; -- sweeper in ringup state
        sweeper_idle : in  STD_LOGIC; -- sweeper in idle state
        distance: out STD_LOGIC_VECTOR (24 downto 0); -- Output: distance
        addr : out STD_LOGIC_VECTOR (14 downto 0);
        we : out  STD_LOGIC;
        recording : OUT STD_LOGIC
    );
end RecordControl;

architecture Behavioral of RecordControl is

    type STATE_T is (IDLE_S, DELAY, RECORD_TRACE, DONE);
    
    signal current_state: STATE_T := IDLE_S;

    -- signal distance_data: STD_LOGIC_VECTOR(24 downto 0) := (others => '0');
    signal internal_counter : STD_LOGIC_VECTOR (31 downto 0) := (others => '0');
    signal addr_sig : STD_LOGIC_VECTOR (14 downto 0) := (others => '0');
    signal we_sig : STD_LOGIC := '0';
    signal max_addr : STD_LOGIC_VECTOR (14 downto 0) := (others => '1');
    signal laser_data_ready_old : STD_LOGIC := '0';
	 signal recording_sig : STD_LOGIC := '0';
    
    
    begin
    
    ready_prog: process(clk100)
    begin
        if rising_edge(clk100) then
            laser_data_ready_old <= laser_data_ready;
        end if;
    
    end process;
    
    
    update_proc: process(clk100)
    begin
        if rising_edge(clk100) then
            case current_state is
                when IDLE_S =>
                recording_sig <= '0';
                internal_counter <= (others => '0');
                we_sig <= '0';
                addr_sig <= (others => '0');
                    if is_ringup = '1' then
                        current_state <= DELAY;
                    else
                        current_state <= current_state;
                    end if;
                when DELAY =>
                recording_sig <= '0';
                we_sig <= '0';
                addr_sig <= (others => '0');
                    if (internal_counter = max_count) then
                        current_state <= RECORD_TRACE;
                        internal_counter <= internal_counter; 
                    else
                        internal_counter <= internal_counter + 1;
                        current_state <= current_state;
                    end if;
                when RECORD_TRACE =>
                internal_counter <= internal_counter;
                    if addr_sig = max_addr then
                        recording_sig <= recording_sig;
                        addr_sig <= addr_sig;
                        current_state <= DONE;
                        we_sig <= '0';
                    else -- fill BRAM, first datum is invalid! (addr = 0)
                        current_state <= current_state;
                        if (laser_data_ready_old = '0') AND (laser_data_ready = '1') then
                            recording_sig <= '1';
                            we_sig <= '1';
                            addr_sig <= addr_sig + 1;
                        else
                            recording_sig <= recording_sig;
                            we_sig <= '0';
                            addr_sig <= addr_sig;
                        end if;
                    end if;
                when DONE =>
                recording_sig <= recording_sig;
                internal_counter <= internal_counter;
                we_sig <= '0';
                addr_sig <= addr_sig;
                    if sweeper_idle = '1' then
                        current_state <= IDLE_S;
                    else
                        current_state <= current_state;
                    end if;
  
            
            end case;
        end if;
    end process;
    
    -- distance <= distance_data;
    distance <= laser_data;
    addr <= addr_sig;
    we <= we_sig;
	 recording <= recording_sig;
    
    end Behavioral;