----------------------------------------------------------------------------------
-- Company: ETH Zurich
-- Engineer: Pascal Engeler <engeler.pascal@gmail.com>
-- 
-- Create Date:    09:31:23 08/21/2024 
-- Design Name: 
-- Module Name:    SerialTransmitter - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
-- We use 1 stop bit and 1 start bit between bytes. Nominally, we want to transmit 20 bytes per frame, and in total each byte corresponds to transmission of 10 bits, such that we need 200/201 bit buffers.
-- The number of CLK100 ticks between DR going low to transmission finished is less than 210 * 870 = 182700 = 0b101100100110101100 (18 digits), where 870 is > the serial_counter_max value determined by the transmission baud rate
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

entity SerialTransmitter is
    Port ( CLK100 : in  STD_LOGIC;
           DR : in  STD_LOGIC;
           --Data : in  STD_LOGIC_VECTOR (7 downto 0);
           Data : in STD_LOGIC_VECTOR(199 downto 0);
           IDLE : out STD_LOGIC;
           TX : out  STD_LOGIC);
end SerialTransmitter;

architecture Behavioral of SerialTransmitter is

signal SERIAL_COUNTER: STD_LOGIC_VECTOR (9 downto 0) := (others => '0');
--signal BUFF: STD_LOGIC_VECTOR(8 downto 0) := (others => '1');
signal BUFF: STD_LOGIC_VECTOR(200 downto 0) := (others => '1');
--baud rate: 2000000 (115200)
--signal serial_counter_max : STD_LOGIC_VECTOR(9 downto 0) := "1101100011"; --change this for baud rate change
signal serial_counter_max : STD_LOGIC_VECTOR(9 downto 0) := "0000110010"; --change this for baud rate change
signal transmission_timer : STD_LOGIC_VECTOR(17 downto 0) := (others => '0');
--signal transmission_timer_max : STD_LOGIC_VECTOR(17 downto 0) := "101100100110101100";
signal transmission_timer_max : STD_LOGIC_VECTOR(17 downto 0) := "000010101101111010";
signal dr_prev: STD_LOGIC := '0';
signal idle_s: STD_LOGIC := '0';

begin

TX <= BUFF(0);
IDLE <= idle_s;

idleCount: process(CLK100)
begin
    if rising_edge(CLK100) then
        dr_prev <= DR;
        if DR = '0' then
            if dr_prev = '1' then
                transmission_timer <= (others => '0');
                idle_s <= '0';
            else
                transmission_timer <= transmission_timer + 1;
                if transmission_timer = transmission_timer_max then
                    idle_s <= '1';
                else
                    idle_s <= idle_s;
                end if;
            end if;
        end if;
    end if;
end process;

processTransmit: process(CLK100)
begin
	if rising_edge(CLK100) then
		if DR = '1' then
			BUFF(200 downto 1) <= Data;
			BUFF(0) <= '0';
			SERIAL_COUNTER <= (others => '0');
		else
			--if SERIAL_COUNTER = "10011" then
            if SERIAL_COUNTER = serial_counter_max then
				SERIAL_COUNTER <= (others => '0');
				--BUFF(7 downto 0) <= BUFF(8 downto 1);
				--BUFF(8) <= '1';
				BUFF(199 downto 0) <= BUFF(200 downto 1);
				BUFF(200) <= '1';
			else
				SERIAL_COUNTER <= SERIAL_COUNTER + 1;
				BUFF <= BUFF;
			end if;
		end if;
	end if;
end process;

end Behavioral;

