----------------------------------------------------------------------------------
-- Company: ETH Zurich
-- Engineer: Pascal Engeler <engeler.pascal@gmail.com>
-- 
-- Create Date:    15:48:33 07/04/2024 
-- Design Name: 
-- Module Name:    Sweeper - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

entity Sweeper is
    Port ( clk100: in STD_LOGIC;
			  ddphi_mem_rd_data : in  STD_LOGIC_VECTOR (22 downto 0);
           dphi_mem_rd_data : in  STD_LOGIC_VECTOR (22 downto 0);
           n_ringup_mem_rd_data : in  STD_LOGIC_VECTOR (31 downto 0);
           n_meas_mem_rd_data : in  STD_LOGIC_VECTOR (31 downto 0);
           n_sweep_mem_rd_data : in  STD_LOGIC_VECTOR (31 downto 0);
           n_freqs : in  STD_LOGIC_VECTOR (9 downto 0);
           start : in  STD_LOGIC;
           lockin_idle : in  STD_LOGIC;
           ddphi_mem_rd_addr : out  STD_LOGIC_VECTOR (9 downto 0);
           dphi_mem_rd_addr : out  STD_LOGIC_VECTOR (9 downto 0);
           n_ringup_mem_rd_addr : out  STD_LOGIC_VECTOR (9 downto 0);
           n_meas_mem_rd_addr : out  STD_LOGIC_VECTOR (9 downto 0);
           n_sweep_mem_rd_addr : out  STD_LOGIC_VECTOR (9 downto 0);
           phi : out  STD_LOGIC_VECTOR (31 downto 0);
           amp_mem_addr : out  STD_LOGIC_VECTOR (9 downto 0);
           samples_mem_addr : out  STD_LOGIC_VECTOR (9 downto 0);
           lockin_data_wr_en : out  STD_LOGIC;
           lockin_en : out  STD_LOGIC;
           lockin_clear : out  STD_LOGIC;
           idle : out  STD_LOGIC;
		   is_ringup: out STD_LOGIC);
end Sweeper;

architecture Behavioral of Sweeper is
--28 non-idle states
type STATE_T is ( STATE_IDLE, STATE_01, STATE_02, STATE_03, STATE_04, STATE_05, STATE_05a, STATE_06, STATE_07, STATE_08, STATE_09, STATE_10,
						STATE_11, STATE_12, STATE_13, STATE_14, STATE_15, STATE_16, STATE_17, STATE_18, STATE_19, STATE_20,
						STATE_21, STATE_22, STATE_23, STATE_24, STATE_25, STATE_26, STATE_27, STATE_28, 
						STATE_12_1, STATE_12_2, STATE_12_3);

signal current_state: STATE_T := STATE_IDLE;

signal ddphi_mem_rd_data_sig: STD_LOGIC_VECTOR(22 downto 0);
signal dphi_mem_rd_data_sig: STD_LOGIC_VECTOR(22 downto 0);
signal n_ringup_mem_rd_data_sig: STD_LOGIC_VECTOR(31 downto 0);
signal n_meas_mem_rd_data_sig: STD_LOGIC_VECTOR(31 downto 0);
signal n_sweep_mem_rd_data_sig: STD_LOGIC_VECTOR(31 downto 0);
signal n_freqs_sig: STD_LOGIC_VECTOR(9 downto 0);
signal start_sig: STD_LOGIC;
signal lockin_idle_sig: STD_LOGIC;


signal ddphi_mem_rd_addr_sig: STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
signal dphi_mem_rd_addr_sig: STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
signal n_ringup_mem_rd_addr_sig: STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
signal n_meas_mem_rd_addr_sig: STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
signal n_sweep_mem_rd_addr_sig: STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
signal phi_sig: STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
signal amp_mem_addr_sig: STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
signal samples_mem_addr_sig: STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
signal lockin_data_wr_en_sig: STD_LOGIC := '0';
signal lockin_en_sig: STD_LOGIC := '0';
signal lockin_clear_sig: STD_LOGIC := '0';
signal idle_sig: STD_LOGIC := '1';
signal is_ringup_sig: STD_LOGIC := '0';

signal current_dphi: STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
signal current_ddphi: STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
signal current_n_ringup: STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
signal current_n_meas: STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
signal current_n_sweep: STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
signal target_dphi: STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
signal ddphi_counter: STD_LOGIC_VECTOR(23 downto 0) := (others => '0');

signal counter: STD_LOGIC_VECTOR(31 downto 0) := (others => '0');

signal old_start: STD_LOGIC := '0';

signal phi_passes_through_zero: STD_LOGIC := '0';

begin

phi_update: process(clk100)
begin
	if rising_edge(clk100) then
		if current_state /= STATE_IDLE then
			if phi_sig(31) = '1' then
			--transition - to +, phi(30 downto 28) must be 0
				phi_sig(31) <= '0';
				phi_sig(30 downto 0) <= phi_sig(30 downto 0) + current_dphi(30 downto 0);
				phi_passes_through_zero <= '1';
			elsif phi_sig(28) = '1' and phi_sig(30) = '0' then
			--transition overflow + to -
				phi_sig(31) <= '0';
				phi_sig(30 downto 28) <= "111";
				phi_sig(27 downto 0) <= phi_sig(27 downto 0) + current_dphi(27 downto 0);
				phi_passes_through_zero <= '0';
			else
			--no transition
				phi_sig <= phi_sig + current_dphi;
				phi_passes_through_zero <= '0';
			end if;
		else
			phi_sig <= (others => '0');
			phi_passes_through_zero <= '0';
		end if;
	end if;
end process;

start_update: process(clk100)
begin
	if rising_edge(clk100) then
		old_start <= start_sig;
	end if;
end process;



--stale signals
--ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
--dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
--n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
--n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
--n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
--amp_mem_addr_sig <= amp_mem_addr_sig;
--samples_mem_addr_sig <= samples_mem_addr_sig;
--lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
--lockin_en_sig <= lockin_en_sig;
--lockin_clear_sig <= lockin_clear_sig;
--idle_sig <= idle_sig;
--current_state <= current_state;
--current_dphi <= current_dphi;
--current_ddphi <= current_ddphi;

state_update: process(clk100)
begin
	if rising_edge(clk100) then
		case current_state is
		when STATE_IDLE =>
			if old_start = '0' AND start_sig = '1' then
				current_state <= STATE_01;
				idle_sig <= '0';
			else
				current_state <= current_state;
				idle_sig <= '1';
			end if;
			--stale signals
			ddphi_mem_rd_addr_sig <= (others => '0');
			dphi_mem_rd_addr_sig <= (others => '0');
			n_ringup_mem_rd_addr_sig <= (others => '0');
			n_meas_mem_rd_addr_sig <= (others => '0');
			n_sweep_mem_rd_addr_sig <= (others => '0');
			amp_mem_addr_sig <= (others => '0');
			samples_mem_addr_sig <= (others => '0');
			lockin_data_wr_en_sig <= '0';
			lockin_en_sig <= '0';
			lockin_clear_sig <= '1';
			current_dphi <= (others => '0');
			current_ddphi <= (others => '0');
			current_n_meas <= (others => '0');
			current_n_ringup <= (others => '0');
			current_n_sweep <= (others => '0');
			counter <= (others => '0');
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
				
		when STATE_01 =>
		--apply addresses
			dphi_mem_rd_addr_sig <= (others => '0');
			n_meas_mem_rd_addr_sig <= (others => '0');
			n_ringup_mem_rd_addr_sig <= (others => '0');
			current_state <= STATE_02;
			idle_sig <= '0';
			--stale signals
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			--dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			--n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			--n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= '0';
			lockin_en_sig <= '0';
			lockin_clear_sig <= '0';
			current_state <= STATE_02;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= (others => '0');
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
			
		when STATE_02 =>
		--wait
			current_state <= STATE_03;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= (others => '0');
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_03 =>
		--wait
			current_state <= STATE_04;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= (others => '0');
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_04 =>
		--latch data
			current_state <= STATE_05;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi(22 downto 0) <= dphi_mem_rd_data_sig(22 downto 0);
			current_ddphi <= current_ddphi;
			current_n_meas(31 downto 0) <= n_meas_mem_rd_data_sig(31 downto 0);
			current_n_ringup(31 downto 0) <= n_ringup_mem_rd_data_sig(31 downto 0);
			current_n_sweep <= current_n_sweep;
			counter <= (others => '0');
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_05 =>
		--ringup
			if counter = current_n_ringup then
				current_state <= STATE_05a;
				counter <= (others => '0');
			else
				current_state <= current_state;
				counter <= counter + 1;
			end if;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= '1'; -- add also to other states! TODO
		
		when STATE_05a =>
		--wait for phase to go through 0
			if phi_passes_through_zero = '1' then
				current_state <= STATE_06;
			else
				current_state <= STATE_05a;
			end if;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= '0';
			lockin_clear_sig <= '0';
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= (others => '0');
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= '0';
		
		when STATE_06 =>
		--enable lockin
			current_state <= STATE_07;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= '1';
			lockin_clear_sig <= '0';
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= (others => '0');
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_07 =>
		--measure
			if counter = current_n_meas then
				current_state <= STATE_08;
				counter <= (others => '0');
			else
				current_state <= current_state;
				counter <= counter + 1;
			end if;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_08 =>
		--enable lockin
			current_state <= STATE_09;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= '0';
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= (others => '0');
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_09 =>
		--apply write addresses (already done)
			current_state <= STATE_10;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_10 =>
		--wait
			current_state <= STATE_11;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_11 =>
		--wait
			current_state <= STATE_12;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_12 =>
		--wait
			current_state <= STATE_12_1;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_12_1 =>
		--wait
			current_state <= STATE_12_2;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_12_2 =>
		--wait
			current_state <= STATE_12_3;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_12_3 =>
		--wait
			current_state <= STATE_13;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_13 =>
		--lockin data write enable up
			current_state <= STATE_14;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= '1';
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_14 =>
		--lockin data write enable down
			current_state <= STATE_15;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= '0';
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_15 =>
		--wait
			current_state <= STATE_16;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_16 =>
		--wait
			current_state <= STATE_17;
			amp_mem_addr_sig <= amp_mem_addr_sig+1;
			samples_mem_addr_sig <= samples_mem_addr_sig+1;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_17 =>
		--lockin clear up
			current_state <= STATE_18;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= '1';
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_18 =>
		--lockin clear down, short circuit exit if done
			if amp_mem_addr_sig = n_freqs_sig then
				current_state <= STATE_IDLE;
			else
				current_state <= STATE_19;
			end if;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= '0';
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_19 =>
		--increment read addresses
			current_state <= STATE_20;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig+1;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig+1;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig+1;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig+1;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig+1;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_20 =>
		--wait
			current_state <= STATE_21;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_21 =>
		--wait
			current_state <= STATE_22;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_22 =>
		--apply read addresses (already done, just wait)
			current_state <= STATE_23;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_23 =>
		--wait
			current_state <= STATE_24;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_24 =>
		--wait
			current_state <= STATE_25;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_25 =>
		--latch dphi, ddphi, nmeas, nringup, nsweep
			current_state <= STATE_26;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			target_dphi(22 downto 0) <= dphi_mem_rd_data_sig;
			current_dphi <= current_dphi;
			current_ddphi(22 downto 0) <= ddphi_mem_rd_data_sig;
			current_n_meas <= n_meas_mem_rd_data_sig;
			current_n_ringup <= n_ringup_mem_rd_data_sig;
			current_n_sweep <= n_sweep_mem_rd_data_sig;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_26 =>
		--wait
			current_state <= STATE_27;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			target_dphi <= target_dphi;
			current_dphi <= current_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= (others => '0');
			ddphi_counter <= (others => '0');
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_27 =>
		--ramp
			if current_dphi = target_dphi then
				current_state <= STATE_28;
				current_dphi <= target_dphi;
				ddphi_counter <= (others => '0');
			elsif ddphi_counter(23) = '1' then
				current_state <= STATE_27;
				ddphi_counter(22 downto 0) <= ddphi_counter(22 downto 0) + current_ddphi(22 downto 0);
				ddphi_counter(23) <= '0';
				current_dphi <= current_dphi + 1;
			else
				current_state <= STATE_27;
				current_dphi <= current_dphi;
				ddphi_counter(23 downto 0) <= ddphi_counter(23 downto 0) + current_ddphi(23 downto 0);
			end if;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			target_dphi <= target_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			is_ringup_sig <= is_ringup_sig;
		
		when STATE_28 =>
		--apply
			current_state <= STATE_05;
			ddphi_mem_rd_addr_sig <= ddphi_mem_rd_addr_sig;
			dphi_mem_rd_addr_sig <= dphi_mem_rd_addr_sig;
			n_ringup_mem_rd_addr_sig <= n_ringup_mem_rd_addr_sig;
			n_meas_mem_rd_addr_sig <= n_meas_mem_rd_addr_sig;
			n_sweep_mem_rd_addr_sig <= n_sweep_mem_rd_addr_sig;
			amp_mem_addr_sig <= amp_mem_addr_sig;
			samples_mem_addr_sig <= samples_mem_addr_sig;
			lockin_data_wr_en_sig <= lockin_data_wr_en_sig;
			lockin_en_sig <= lockin_en_sig;
			lockin_clear_sig <= lockin_clear_sig;
			idle_sig <= idle_sig;
			current_dphi <= target_dphi;
			current_ddphi <= current_ddphi;
			current_n_meas <= current_n_meas;
			current_n_ringup <= current_n_ringup;
			current_n_sweep <= current_n_sweep;
			counter <= counter;
			ddphi_counter <= ddphi_counter;
			target_dphi <= target_dphi;
			is_ringup_sig <= is_ringup_sig;
		
		end case;
	end if;
end process;



ddphi_mem_rd_data_sig <= ddphi_mem_rd_data;
dphi_mem_rd_data_sig <= dphi_mem_rd_data;
n_ringup_mem_rd_data_sig <= n_ringup_mem_rd_data;
n_meas_mem_rd_data_sig <= n_meas_mem_rd_data;
n_sweep_mem_rd_data_sig <= n_sweep_mem_rd_data;
n_freqs_sig <= n_freqs;
start_sig <= start;
lockin_idle_sig <= lockin_idle;

ddphi_mem_rd_addr <= ddphi_mem_rd_addr_sig;
dphi_mem_rd_addr <= dphi_mem_rd_addr_sig;
n_ringup_mem_rd_addr <= n_ringup_mem_rd_addr_sig;
n_meas_mem_rd_addr <= n_meas_mem_rd_addr_sig;
n_sweep_mem_rd_addr <= n_sweep_mem_rd_addr_sig;
--sanitize phi
phi(31) <= phi_sig(28);
phi(30) <= phi_sig(28);
phi(29) <= phi_sig(28);
phi(28) <= phi_sig(28);
phi(27 downto 0) <= phi_sig(27 downto 0);
amp_mem_addr <= amp_mem_addr_sig;
samples_mem_addr <= samples_mem_addr_sig;
lockin_data_wr_en <= lockin_data_wr_en_sig;
lockin_en <= lockin_en_sig;
lockin_clear <= lockin_clear_sig;
idle <= idle_sig;
is_ringup <= is_ringup_sig;



end Behavioral;

